@echo off

setlocal EnableDelayedExpansion

set "startTime=%time: =0%"

rem The CMake generator and other flags can be passed as parameters.
set CMAKE_FLAGS=-G "Visual Studio 15 Win64"
IF NOT "%~1"=="" (
  SET CMAKE_FLAGS=%*
)

IF "%NGT_ITD_DEBUG_BUILD%"=="true" (
  echo Debug build is enabled!
  set BUILD_TYPE=Debug
) else (
  set BUILD_TYPE=Release
)

rem This directory should contain the top-level CMakeLists.txt - it is assumed to reside in the same
rem directory as this script.
set CMAKE_DIR=%~dp0


rem Get the current directory - this is the default location for the build and install directory.
set CURRENT_DIR=%cd%

rem The build directory.
set BUILD_DIR=%CURRENT_DIR%\build\windows-%BUILD_TYPE%

rem The install directory.
set INSTALL_DIR=%CURRENT_DIR%/install/windows-%BUILD_TYPE%

rem create build directory if necessary -----------------------------------------------------------

if exist %BUILD_DIR% goto BUILD_DIR_CREATED
    mkdir %BUILD_DIR%
:BUILD_DIR_CREATED

rem configure, compile & install ------------------------------------------------------------------

cd %BUILD_DIR% || goto :failure
cmake %CMAKE_FLAGS% -DCMAKE_INSTALL_PREFIX=%INSTALL_DIR% -DCMAKE_BUILD_TYPE=%BUILD_TYPE% %CMAKE_DIR% || goto :failure
cmake --build . --config %BUILD_TYPE% --target install --parallel %NUMBER_OF_PROCESSORS% || goto :failure

rem -----------------------------------------------------------------------------------------------

:success
echo Finished successfully.
echo """"""""""""""""""""""

set "endTime=%time: =0%"

rem Get elapsed time:
set "end=!endTime:%time:~8,1%=%%100)*100+1!"  &  set "start=!startTime:%time:~8,1%=%%100)*100+1!"
set /A "elap=((((10!end:%time:~2,1%=%%100)*60+1!%%100)-((((10!start:%time:~2,1%=%%100)*60+1!%%100), elap-=(elap>>31)*24*60*60*100"

rem Convert elapsed time to HH:MM:SS:CC format:
set /A "cc=elap%%100+100,elap/=100,ss=elap%%60+100,elap/=60,mm=elap%%60+100,hh=elap/60+100"

echo Start:    %startTime%
echo End:      %endTime%
echo Elapsed:  %hh:~1%%time:~2,1%%mm:~1%%time:~2,1%%ss:~1%%time:~8,1%%cc:~1%

goto :end

:failure
echo Errors occurred! 1>&2

:end
cd %CURRENT_DIR%

@echo on