Param (
    [Parameter(ValueFromRemainingArguments)]
    [string[]]$CMakeArgs,

    [string]$Generator = 'Visual Studio 15 Win64',

    [string]$BuildType = 'Release',

    [string]$Qt5Root = $env:Qt5_ROOT_DIR
)

if (($Qt5Root -eq "") -or ($Qt5Root -eq $null)) {
    $Qt5Root = Read-Host -Prompt "Please enter the Qt5 root directory: ";
}
$Qt5Root = $Qt5Root -replace '[\\]', '/';

Write-Host;
Write-Host "      Generator: $Generator";
Write-Host "     Build Type: $BuildType";
Write-Host "CMake Arguments: $CMakeArgs";
Write-Host "Qt Installation: $Qt5Root";
Write-Host;

$CMakeDirectory = $PSScriptRoot -replace '[\\]', '/';
$CurrentDirectory = (Get-Location).path -replace '[\\]', '/';

$BuildDirectory = "$CurrentDirectory/build/windows-$BuildType";
$InstallDirectory = "$CurrentDirectory/install/windows-$BuildType";

cmake -E make_directory "$BuildDirectory";
Set-Location -Path "$BuildDirectory";

Try
{
    $sw = [Diagnostics.Stopwatch]::StartNew();

    cmake -G "$Generator" -DCMAKE_INSTALL_PREFIX="$InstallDirectory" -DCMAKE_BUILD_TYPE="$BuildType" -DQt5_DIR="$Qt5Root/lib/cmake/Qt5" $CMakeArgs "$CMakeDirectory";

    If ($LASTEXITCODE -ne 0)
    {
        Throw "Error while configuring CMake! Error Code: $LASTEXITCODE";
    }

    cmake --build . --config "$BuildType" --target install --parallel $ENV:NUMBER_OF_PROCESSORS;

    If ($LASTEXITCODE -ne 0)
    {
        Throw "Error while building the application! Error Code: $LASTEXITCODE";
    }

    $sw.Stop();
    $t = $sw.Elapsed;

    Write-Host ([string]::Format("Elapsed Time: {0:00}:{1:00}:{2:00} (hh:mm:ss)", $t.Hours, $t.Minutes, $t.Seconds));
    Write-Host "Finished.";
}
Finally
{
    Set-Location -Path "$CurrentDirectory";
}
