#!/bin/bash

# The CMake generator and other flags can be passed as parameters.
CMAKE_FLAGS=(-G "Unix Makefiles")
if [ $# -ne 0 ]; then
  CMAKE_FLAGS=( "$@" )
fi

BUILD_TYPE=Release
case "$NGT_ITD_DEBUG_BUILD" in
  (true) echo "Debug build is enabled!"; BUILD_TYPE=Debug;
esac

CMAKE_DIR="$( cd "$( dirname "$0" )" && pwd )"

# Get the current directory - this is the default location for the build and install directory.
CURRENT_DIR="$(pwd)"

# The build directory.
BUILD_DIR="$CURRENT_DIR/build/linux-$BUILD_TYPE"

# The install directory.
INSTALL_DIR="$CURRENT_DIR/install/linux-$BUILD_TYPE"

# create the build directory if necessary ----------------------------------------------------------
if [ ! -d "$BUILD_DIR" ]; then
  mkdir -p "$BUILD_DIR"
fi

# configure, compile & install ---------------------------------------------------------------------

cd "$BUILD_DIR" || exit
cmake "${CMAKE_FLAGS[@]}" -DCMAKE_INSTALL_PREFIX="$INSTALL_DIR" -DCMAKE_BUILD_TYPE=$BUILD_TYPE "$CMAKE_DIR"
cmake --build . --target install --parallel "$(nproc)"

# Delete empty files installed by cmake
find "$INSTALL_DIR" -type d -empty -delete
