#! /bin/bash

BUILD_TYPE=Release
case "$NGT_ITD_DEBUG_BUILD" in
  (true) echo "Debug build is enabled!"; BUILD_TYPE=Debug; export EMCC_DEBUG=1;
esac

CMAKE_DIR="$( cd "$( dirname "$0" )" && pwd )"

# Get the current directory - this is the default location for the build and install directory.
CURRENT_DIR="$(pwd)"

# The build directory.
BUILD_DIR="$CURRENT_DIR/build/wasm-$BUILD_TYPE"

# The install directory.
INSTALL_DIR="$CURRENT_DIR/install/wasm-$BUILD_TYPE"

# create the build directory if necessary ----------------------------------------------------------
if [ ! -d "$BUILD_DIR" ]; then
  mkdir -p "$BUILD_DIR"
fi

QTC="$QT5_WASM_ROOT/lib/cmake"

# configure, compile & install ---------------------------------------------------------------------
cd "$BUILD_DIR" || exit

emcmake cmake -DCMAKE_INSTALL_PREFIX="$INSTALL_DIR" -DCMAKE_BUILD_TYPE="$BUILD_TYPE" \
                  -DQt5_DIR="$QTC/Qt5" \
                  -DQt5Core_DIR="$QTC/Qt5Core" \
                  -DQt5Quick_DIR="$QTC/Qt5Quick" \
                  -DQt5QmlModels_DIR="$QTC/Qt5QmlModels" \
                  -DQt5Qml_DIR="$QTC/Qt5Qml" \
                  -DQt5Network_DIR="$QTC/Qt5Network" \
                  -DQt5Gui_DIR="$QTC/Qt5Gui" \
                  -DQt5Svg_DIR="$QTC/Qt5Svg" \
                  -DQt5Widgets_DIR="$QTC/Qt5Widgets" \
                  -DQt5Zlib_DIR="$QTC/Qt5Zlib" \
                  -DQt5EventDispatcherSupport_DIR="$QTC/Qt5EventDispatcherSupport" \
                  -DQt5FontDatabaseSupport_DIR="$QTC/Qt5FontDatabaseSupport" \
                  -DQt5EglSupport_DIR="$QTC/Qt5EglSupport" \
                  -DQt5QuickControls2_DIR="$QTC/Qt5QuickControls2" \
                  -DQt5Charts_DIR="$QTC/Qt5Charts" \
                  -DQt5QmlImportScanner_DIR="$QTC/Qt5QmlImportScanner" \
                  "$CMAKE_DIR"
                  
emmake make install -j$(nproc)
