const MAX_DECELERATION = 0.8 // m/s^2

const EMPTY_MASS = 384000 // kg
const AVERAGE_PASSENGER_WEIGHT = 75 // kg
const NAM_PASSENGERS = 800 // DIN-Human
const TOTAL_MASS = EMPTY_MASS + NAM_PASSENGERS * AVERAGE_PASSENGER_WEIGHT // kg

const davisCoefficientA = 5499.878 // N
const davisCoefficientB = 0 // N / (m/s)
const davisCoefficientC = 7.936942 // N / (m/s)^2

/**
 * @param power{W}
 * @param mass{kg}
 * @param velocity{m/s}
 * @returns {number}
 */
function acceleration(power, mass, velocity) {
   return (power
      + davisCoefficientA * velocity
      + davisCoefficientB * velocity * velocity
      + davisCoefficientC * velocity * velocity * velocity)
      / (mass * velocity);
}

/**
 * @param power{W}
 * @param mass{kg}
 * @param velocity{m/s}
 * @returns {number}
 */
function deceleration(power, mass, velocity) {
   return -Math.min(acceleration(power, mass, velocity), MAX_DECELERATION);
}

/**
 * @param speed{m/s}
 * @param speedInaccuracy{m/s}
 * @param delay{s}
 * @returns {number}
 */
function constantSpeedDistance(speed, speedInaccuracy, delay) {
   return (speed + speedInaccuracy) * delay;
}

const STEP_SIZE = 0.01;

/**
 * @param power{W}
 * @param mass{kg}
 * @param velocity{m/s}
 * @param velocityInaccuracy{m/s}
 * @param positionInaccuracy{m}
 * @returns {{positions: m[], speeds: m/s[]}}
 */
function calculateBreakCurve(power, mass, velocity, velocityInaccuracy = 0, positionInaccuracy = 0) {
   let currentSpeed = velocity + velocityInaccuracy;
   let currentPosition = 0;

   const positions = [currentPosition];
   const speeds = [currentSpeed];

   while (currentSpeed > 0.0) {
      const currentDeceleration = deceleration(power, mass, currentSpeed);

      currentSpeed += currentDeceleration * STEP_SIZE;
      currentPosition += currentSpeed * STEP_SIZE;

      positions.push(currentPosition);
      speeds.push(currentSpeed);
   }

   const endPosition = positions[positions.length - 1];
   for (let i = 0; i < positions.length; ++i) {
      positions[i] = (endPosition - positions[i]) + positionInaccuracy;
   }

   return {positions: positions, speeds: speeds};
}