import QtQuick 2.15
import QtQuick.Controls 2.14
import QtQuick.Layouts 1.14

import "../theme"

Rectangle {
   id: infoPopup

   property string tooltipText: ""

   enum Position {
      TopLeft,
      Top,
      TopRight,
      Right,
      BottomRight,
      Bottom,
      BottomLeft,
      Left
   }

   property var popupPosition: InfoPopup.Position.Right

   visible: tooltipText !== "";

   anchors.verticalCenter: parent.verticalCenter
   width: 24
   height: 24
   radius: 12
   color: DLRTheme.teaserBlue
   border.color: "white"

   MouseArea {
      id: infoArea
      anchors.fill: parent
      hoverEnabled: true

      Component.onCompleted: if (!appInfo.isWeb()) { infoArea.cursorShape = Qt.WhatsThisCursor; }

      onEntered: {
         tooltip.visible = true;
         tooltip.text = tooltipText;

         const rect = infoPopup.mapToItem(frame, 0, 0, infoPopup.width, infoPopup.height);

         const xLeft = rect.x - tooltip.width;
         const xMiddle = rect.x + ((rect.width - tooltip.width) / 2);
         const xRight = rect.x + rect.width;

         const yTop = rect.y - tooltip.height;
         const yMiddle = rect.y + ((rect.height - tooltip.height) / 2);
         const yBottom = rect.y + rect.height;

         switch (popupPosition) {
            case InfoPopup.Position.TopLeft:
               tooltip.transformOrigin = Item.BottomRight;
               tooltip.x = xLeft;
               tooltip.y = yTop;
               break;

            case InfoPopup.Position.Top:
               tooltip.transformOrigin = Item.Bottom;
               tooltip.x = xMiddle;
               tooltip.y = yTop;
               break;

            case InfoPopup.Position.TopRight:
               tooltip.transformOrigin = Item.BottomLeft;
               tooltip.x = xRight;
               tooltip.y = yTop;
               break;

            case InfoPopup.Position.Right:
               tooltip.transformOrigin = Item.Left;
               tooltip.x = xRight;
               tooltip.y = yMiddle;
               break;

            case InfoPopup.Position.BottomRight:
               tooltip.transformOrigin = Item.TopLeft;
               tooltip.x = xRight;
               tooltip.y = yBottom;
               break;

            case InfoPopup.Position.Bottom:
               tooltip.transformOrigin = Item.Top;
               tooltip.x = xMiddle;
               tooltip.y = yBottom;
               break;

            case InfoPopup.Position.BottomLeft:
               tooltip.transformOrigin = Item.TopRight;
               tooltip.x = xLeft;
               tooltip.y = yBottom;
               break;

            case InfoPopup.Position.Left:
               tooltip.transformOrigin = Item.Right;
               tooltip.x = xLeft;
               tooltip.y = yMiddle;
               break;
         }
      }

      onExited: {
         tooltip.visible = false;
         tooltip.text = "";
         tooltip.x = 0;
         tooltip.y = 0;
      }
   }

   DLRLabel {
      anchors.centerIn: parent
      text: "?"
      font.pointSize: 14
      font.bold: true
      color: "white"
   }
}