import QtQuick 2.14
import QtQuick.Controls 2.14
import QtQuick.Layouts 1.14
import QtQuick.Extras 1.4
import QtGraphicalEffects 1.12
import QtQuick.Shapes 1.14

import IO 1.0

import "../theme"

MouseArea {
   id: root

   property string title
   property string descriptionURL

   signal closed()
   signal opened()

   anchors.fill: parent

   onClicked: close()

   function close() {
      root.enabled = false;
      root.visible = false;
      closed();
   }

   function open() {
      root.enabled = true;
      root.visible = true;
      opened();
   }

   Rectangle {
      anchors.fill: parent
      color: "black"
      opacity: 2.0 / 3.0
   }

   Rectangle {
      scale: widgetScale
      transformOrigin: Item.Center
      anchors.centerIn: parent
      width: 0.5 * app.referenceWidth
      height: 0.75 * app.referenceHeight

      radius: 10
      border.width: 2
      border.color: "black"
      color: DLRTheme.backgroundLightGrey

      DLRLabel {
         id: descriptionTitle

         anchors {
            top: parent.top
            topMargin: 15
            horizontalCenter: parent.horizontalCenter
         }

         font.pointSize: 24
         font.bold: true
         text: root.title
      }

      Rectangle {
         anchors {
            top: descriptionTitle.bottom
            left: parent.left
            right: parent.right
            topMargin: 15
            leftMargin: 15
            rightMargin: 15
         }

         height: 1
         color: "black"
      }

      ScrollView {
         id: textScroll

         anchors {
            top: descriptionTitle.bottom
            left: parent.left
            right: parent.right
            bottom: startButton.top
            margins: 15
         }

         contentWidth: width - 5
         contentHeight: descriptionText.height
         clip: true
         ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
         ScrollBar.vertical.policy: contentHeight > height ? ScrollBar.AlwaysOn : ScrollBar.AlwaysOff


         DLRLabel {
            id: descriptionText
            topPadding: 15
            bottomPadding: 15
            width: parent.width - 5

            textFormat: Text.RichText
            wrapMode: Text.WordWrap
            font.pointSize: 14

            FileReader {
               id: fileReader
            }

            text: fileReader.readFile(root.descriptionURL)
         }
      }

      Rectangle {
         anchors {
            bottom: startButton.top
            left: parent.left
            right: parent.right
            bottomMargin: 15
            leftMargin: 15
            rightMargin: 15
         }

         height: 1
         color: "black"
      }

      DLRButton {
         id: startButton

         anchors {
            bottom: parent.bottom
            horizontalCenter: parent.horizontalCenter
            bottomMargin: 15
         }

         text: "Go!"

         onClicked: root.close()
      }
   }
}