import QtGraphicalEffects 1.0
import QtQuick 2.14
import QtCharts 2.14
import QtQuick.Extras 1.4
import QtQuick.Controls 1.4
import QtQuick.Controls 2.14
import QtQuick.Controls.Styles 1.4
import QtQuick.Layouts 1.14
import QtQuick.Shapes 1.14
import QtQuick.Window 2.15

import "theme"
import "components"
import App 1.0
import IO 1.0

Window {
   id: app
   title: "Next Generation Train (NGT) - Slip Coaching and Virtual Coupling Demonstrator"

   AppInfo {
      id: appInfo
   }

   function toColor(colorString) {
      const red   = parseInt("0x" + colorString.substring(1, 3)) / 255.0;
      const green = parseInt("0x" + colorString.substring(3, 5)) / 255.0;
      const blue  = parseInt("0x" + colorString.substring(5, 7)) / 255.0;
      return Qt.rgba(red, gree, blue, 1);
   }

   property int referenceWidth: 1600
   property int referenceHeight: 900
   property real widgetScale: Math.min(height / referenceHeight, width / referenceWidth)
   property real horizontalMargin: 25 * widgetScale
   property real verticalMargin: 25 * widgetScale

   property bool readOverviewText: false

   Loader {
      id: pageLoader
      source: "overview.qml"
      anchors.fill: parent
      clip: true
   }

   Image {
      id: signet
      source: "../images/DLRSignet.png"
      scale: 0.2 * widgetScale
      transformOrigin: Item.BottomLeft
      mipmap: true
      anchors {
         left: parent.left
         bottom: parent.bottom
         leftMargin: horizontalMargin
         bottomMargin: verticalMargin
      }
   }

   DLRLabel {
      id: copyrightMark

      scale: widgetScale
      transformOrigin: Item.BottomLeft

      anchors {
         bottom: parent.bottom
         left: parent.left
         bottomMargin: 2
         leftMargin: 2
      }

      color: "blue"
      font.underline: true
      font.pointSize: 8

      text: "\u00A9 2021 German Aerospace Center (DLR). Licensed under GNU GPLv3."

      MouseArea {
         anchors.fill: parent
         onClicked: legalTextRoot.open()
      }

      MouseArea {
         anchors.fill: parent
         cursorShape: Qt.PointingHandCursor
         acceptedButtons: Qt.NoButton
         Component.onCompleted: if (appInfo.isWeb()) { visible = false; }
      }
   }

   MouseArea {
      id: legalTextRoot
      anchors.fill: parent
      visible: false

      onClicked: close()

      function close() {
         legalTextRoot.enabled = false;
         legalTextRoot.visible = false;
      }

      FileReader {
         id: fileReader
      }

      function open() {
         legalTextRoot.enabled = true;
         legalTextRoot.visible = true;
         if (legalText.text === "") {
            legalText.text = fileReader.readFile(":data/legal.html");
         }
      }

      Rectangle {
         anchors.fill: parent
         color: "black"
         opacity: 2.0 / 3.0
      }

      Rectangle {
         scale: widgetScale
         transformOrigin: Item.Center
         anchors.centerIn: parent
         width: 0.75 * app.referenceWidth
         height: 0.9 * app.referenceHeight

         radius: 10
         border.width: 2
         border.color: "black"
         color: DLRTheme.backgroundLightGrey

         DLRLabel {
            id: legalTitle
            anchors {
               top: parent.top
               topMargin: 15
               horizontalCenter: parent.horizontalCenter
            }

            font.pointSize: 24
            font.bold: true
            text: "Licenses"
         }

         Rectangle {
            anchors {
               top: legalTitle.bottom
               left: parent.left
               right: parent.right
               topMargin: 15
               leftMargin: 15
               rightMargin: 15
            }

            height: 1
            color: "black"
         }

         ScrollView {
            id: legalTextScroll

            anchors {
               top: legalTitle.bottom
               left: parent.left
               right: parent.right
               bottom: startButton.top
               margins: 15
            }

            contentWidth: width - 5
            contentHeight: legalText.height
            clip: true
            ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
            ScrollBar.vertical.policy: contentHeight > height ? ScrollBar.AlwaysOn : ScrollBar.AlwaysOff


            DLRLabel {
               id: legalText
               topPadding: 15
               bottomPadding: 15
               width: parent.width - 5

               textFormat: Text.RichText
               wrapMode: Text.WordWrap
               font.pointSize: 11

               text: ""
            }
         }

         Rectangle {
            anchors {
               bottom: startButton.top
               left: parent.left
               right: parent.right
               bottomMargin: 15
               leftMargin: 15
               rightMargin: 15
            }

            height: 1
            color: "black"
         }

         DLRButton {
            id: startButton

            anchors {
               bottom: parent.bottom
               horizontalCenter: parent.horizontalCenter
               bottomMargin: 15
            }

            text: "Close"

            onClicked: legalTextRoot.close()
         }
      }
   }

   Shortcut {
      sequence: "Esc"
      onActivated: close()
   }

   property bool isFullscreen: false
   Shortcut {
      sequence: "f"
      onActivated: {
         if (isFullscreen = !isFullscreen) {
            showFullScreen();
         } else {
            showNormal();
         }
      }
   }

   visible: true
   color: DLRTheme.backgroundWhite

   height: 720
   width: 1280

   Component.onCompleted: showMaximized()

   Item {
      id: frame
      anchors.fill: parent
      Rectangle {
         id: tooltip

         scale: widgetScale

         visible: false

         property string text

         z: Number.MAX_VALUE

         width: popupLabel.width + 8
         height: popupLabel.implicitHeight + 6

         color: "white"
         border.color: "black"
         radius: 3

         DLRLabel {
            id: popupLabel
            anchors.centerIn: parent
            text: tooltip.text
            width: 350
            font.pointSize: 12
            textFormat: Text.MarkdownText
            wrapMode: Text.WordWrap
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment : Text.AlignHCenter
         }
      }
   }
}
