import QtQuick 2.15
import QtQuick.Controls 2.14
import QtQuick.Layouts 1.14
import QtQuick.Extras 1.4
import QtGraphicalEffects 1.12
import QtQuick.Shapes 1.14

import Overview 1.0
import IO 1.0
import "overview"
import "overview/components"
import "overview/descriptions.js" as Descriptions
import "components"
import "theme"

Item {
   id: root
   anchors.fill: parent

   OverviewModel {
      id: overviewModel
   }

   Timer {
      interval: 8
      running: true
      repeat: true

      onTriggered: {
         overviewModel.simulate(interval);
      }
   }

   OverviewMap {
      id: overviewMap
   }

   Column {
      id: timeCol
      scale: widgetScale
      transformOrigin: Item.TopRight

      anchors {
         top: parent.top
         right: parent.right
         rightMargin: horizontalMargin
         topMargin: verticalMargin
      }

      width: 0.25 * app.referenceWidth

      ValueSlider {
         id: speedSlider
         width: parent.width

         title: "Simulation Speed"
         unit: "x"

         start: 0
         current: 5
         end: 100
         step: 1

         Binding {
            target: overviewModel
            property: "simulationSpeed"
            value: speedSlider.current
         }
      }

      DLRLabel {
         anchors.right: parent.right

         text: "Zoom: " + overviewMap.touchMap.currentZoom.toFixed(0) + "x"
         font.pointSize: 18
      }
   }

   ButtonGroup {
      id: radioGroup
      exclusive: true
      property int lastScenario: 0;
      onClicked: {
         if (lastScenario !== button.scenario) {
            lastScenario = button.scenario;
            overviewModel.setScenario(button.scenario);
         }
      }
   }

   DLRLabel {
      scale: widgetScale
      transformOrigin: Item.Right

      anchors {
         right: parent.right
         rightMargin: horizontalMargin
         verticalCenter: parent.verticalCenter
      }

      width: contentWidth

      text: "Slip Coaching,\npowered by Virtual Coupling"
      font.pointSize: 25
      font.italic: true
      wrapMode: Text.WordWrap
   }

   Rectangle {
      scale: widgetScale
      transformOrigin: Item.Left
      width: 250
      height: app.referenceHeight / 3

      anchors {
         verticalCenter: parent.verticalCenter
         left: parent.left
         leftMargin: horizontalMargin
      }

      radius: 10
      border.width: 2
      border.color: "black"
      color: DLRTheme.backgroundLightGrey

      DLRLabel {
         id: shortDescription

         anchors {
            fill: parent
            margins: 10
         }

         textFormat: Text.RichText
         wrapMode: Text.WordWrap
         font.pointSize: 11

         text: Descriptions.getTexts(radioGroup.lastScenario).short
      }

      Column {
         anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
            margins: parent.border.width + 4
         }

         spacing: 5

         DLRButton {
            id: showDetailsButton

            anchors {
               left: parent.left
               right: parent.right
            }

            checkable: true
            text: "In Detail"
         }

         DLRButton {
            id: referenceLineButton

            anchors {
               left: parent.left
               right: parent.right
            }

            checkable: true
            text: "Reference Line"
         }
      }
   }

   Rectangle {
      id: navigation
      scale: widgetScale
      transformOrigin: Item.BottomRight
      width: scenariosTitle.width + 30
      height: childrenRect.height + 30

      anchors {
         bottom: parent.bottom
         right: parent.right
         rightMargin: horizontalMargin
         bottomMargin: verticalMargin
      }

      radius: 10
      border.width: 2
      border.color: "black"
      color: DLRTheme.backgroundLightGrey

      Column {
         spacing: 5

         x: 15
         y: 15
         width: scenariosTitle.width
         height: childrenRect.height

         DLRLabel {
            id: scenariosTitle
            font.pointSize: 20
            font.bold: true
            text: "Further Scenarios"
         }

         ZoomButton {
            labelText: "Slip Coaching"
            targetView: "../slip-coaching.qml"
         }

         ZoomButton {
            labelText: "Speed Limits"
            targetView: "../speed-limits.qml"
         }

         ZoomButton {
            labelText: "Brake Curves"
            targetView: "../brake-curves.qml"
         }

         ZoomButton {
            labelText: "Coupling"
            targetView: "../coupling.qml"
         }

         ZoomButton {
            labelText: "Switches"
            targetView: "../switches.qml"
         }
      }
   }

   Rectangle {
      scale: widgetScale
      transformOrigin: Item.BottomLeft
      anchors {
         bottom: parent.bottom
         bottomMargin: verticalMargin
         left: parent.left
         leftMargin: signet.width * 0.2 * widgetScale + 2 * horizontalMargin
      }

      width: app.referenceWidth / 5
      height: app.referenceHeight / 12

      border.color: "black"
      color: DLRTheme.backgroundLightGrey
      radius: 10
      border.width: 2
      Column {
         anchors.centerIn: parent
         spacing: 5

         DLRLabel {
            text: "Legend"
            font.bold: true
         }

         Rectangle {
            width: app.referenceWidth / 6
            height: 1
            color: "black"
         }

         Row {
            anchors.horizontalCenter: parent.horizontalCenter

            Rectangle {
               anchors.verticalCenter: parent.verticalCenter

               color: DLRTheme.teaserGreen
               width: 7
               height: 7
               radius: 3.5
               border.color: "black"
            }

            Rectangle {
               width: 5
               height: 5
               color: "transparent"
            }

            DLRLabel {
               anchors.verticalCenter: parent.verticalCenter
               text: "Train Wagon"
               font.pointSize: 12
            }

            Rectangle {
               width: 20
               height: 5
               color: "transparent"
            }

            Rectangle {
               anchors.verticalCenter: parent.verticalCenter

               color: DLRTheme.teaserYellow
               width: 10
               height: 10
               radius: 10
               border.color: "black"
            }

            Rectangle {
               width: 5
               height: 5
               color: "transparent"
            }

            DLRLabel {
               anchors.verticalCenter: parent.verticalCenter
               text: "Station"
               font.pointSize: 12
            }

            Rectangle {
               width: 20
               height: 5
               color: "transparent"
            }

            Rectangle {
               anchors.verticalCenter: parent.verticalCenter

               color: "white"
               width: 10
               height: 10
               radius: 10
               border.color: "black"
            }

            Rectangle {
               width: 5
               height: 5
               color: "transparent"
            }

            DLRLabel {
               anchors.verticalCenter: parent.verticalCenter
               text: "Switch"
               font.pointSize: 12
            }
         }
      }
   }

   MouseArea {
      anchors.fill: parent
      enabled: showDetailsButton.checked
      visible: showDetailsButton.checked
      onClicked: showDetailsButton.checked = false;

      z: 1000

      Rectangle {
         anchors.fill: parent
         visible: showDetailsButton.checked
         color: "black"
         opacity: 2.0 / 3.0
      }

      Rectangle {
         scale: widgetScale
         transformOrigin: Item.Center
         anchors.centerIn: parent
         width: 0.5 * app.referenceWidth
         height: 0.75 * app.referenceHeight

         radius: 10
         border.width: 2
         border.color: "black"
         color: DLRTheme.backgroundLightGrey

         visible: showDetailsButton.checked

         DLRLabel {
            anchors {
               top: parent.top
               topMargin: 0.25 * verticalMargin
               right: parent.right
               rightMargin: 0.5 * horizontalMargin
            }

            text: "x"
            font {
               pointSize: 32
               bold: false
            }
         }

         DLRLabel {
            id: longDescriptionTitle

            anchors {
               top: parent.top
               topMargin: 15
               horizontalCenter: parent.horizontalCenter
            }

            font.pointSize: 24
            font.bold: true
            text: radioGroup.checkedButton.text
         }

         Rectangle {
            anchors {
               top: longDescriptionTitle.bottom
               left: parent.left
               right: parent.right
               topMargin: 15
               leftMargin: 15
               rightMargin: 15
            }

            height: 1
            color: "black"
         }

         ScrollView {
            anchors {
               top: longDescriptionTitle.bottom
               left: parent.left
               right: parent.right
               bottom: parent.bottom
               margins: 15
            }

            contentWidth: width - 5
            contentHeight: longDescription.height
            clip: true
            ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
            ScrollBar.vertical.policy: contentHeight > height ? ScrollBar.AlwaysOn : ScrollBar.AlwaysOff

            DLRLabel {
               id: longDescription
               topPadding: 15
               width: parent.width - 5

               textFormat: Text.RichText
               wrapMode: Text.WordWrap
               font.pointSize: 14

               FileReader {
                  id: fileReader
               }

               text: fileReader.readFile(Descriptions.getTexts(radioGroup.lastScenario).url)
            }
         }
      }
   }

   MouseArea {
      anchors.fill: parent
      enabled: referenceLineButton.checked
      visible: referenceLineButton.checked
      onClicked: referenceLineButton.checked = false;

      z: 2000

      Rectangle {
         anchors.fill: parent
         visible: referenceLineButton.checked
         color: "black"
         opacity: 2.0 / 3.0
      }

      Rectangle {
         scale: widgetScale
         transformOrigin: Item.Center
         anchors.centerIn: parent
         width: 0.5 * app.referenceWidth
         height: 0.75 * app.referenceHeight

         radius: 10
         border.width: 2
         border.color: "black"
         color: DLRTheme.backgroundLightGrey

         visible: referenceLineButton.checked

         DLRLabel {
            anchors {
               top: parent.top
               topMargin: 0.25 * verticalMargin
               right: parent.right
               rightMargin: 0.5 * horizontalMargin
            }

            text: "x"
            font {
               pointSize: 32
               bold: false
            }
         }

         DLRLabel {
            id: referenceLineTitle
            anchors {
               top: parent.top
               topMargin: 15
               horizontalCenter: parent.horizontalCenter
            }

            font.pointSize: 24
            font.bold: true
            text: "Reference Line"
         }

         Rectangle {
            anchors {
               top: referenceLineTitle.bottom
               left: parent.left
               right: parent.right
               topMargin: 15
               leftMargin: 15
               rightMargin: 15
            }

            height: 1
            color: "black"
         }

         ScrollView {
            anchors {
               top: referenceLineTitle.bottom
               left: parent.left
               right: parent.right
               bottom: parent.bottom
               margins: 15
            }

            contentWidth: width - 5
            contentHeight: referenceLineDescription.height
            clip: true
            ScrollBar.horizontal.policy: ScrollBar.AlwaysOff
            ScrollBar.vertical.policy: contentHeight > height ? ScrollBar.AlwaysOn : ScrollBar.AlwaysOff

            DLRLabel {
               id: referenceLineDescription
               topPadding: 15
               width: parent.width - 5

               textFormat: Text.RichText
               wrapMode: Text.WordWrap
               font.pointSize: 14

               FileReader {
                  id: referenceLineReader
               }

               text: referenceLineReader.readFile(":/gui/overview/reference-line.html")
            }
         }
      }
   }

   DLRButton {
      anchors {
         bottom: parent.bottom
         right: parent.right
         rightMargin: horizontalMargin
         bottomMargin: 1.5 * verticalMargin + navigation.height * widgetScale
      }

      width: navigation.width

      scale: widgetScale
      transformOrigin: Item.BottomRight

      text: "Show Info"
      onClicked: scenarioDescription.open()
   }

   ScenarioDescription {
      id: scenarioDescription
      z: 2000

      visible: !readOverviewText

      title: "Map of Germany"
      descriptionURL: ":/gui/overview/description.html"

      onOpened: speedSlider.current = 0
      onClosed: {
         speedSlider.current = 5;
         readOverviewText = true;
      }
   }

   Column {
      scale: widgetScale
      transformOrigin: Item.TopLeft
      width: 250
      spacing: 5

      z: 1500

      anchors {
         top: parent.top
         left: parent.left
         leftMargin: horizontalMargin
         topMargin: verticalMargin
      }

      DLRButton {
         checked: true
         checkable: true
         text: "Conventional Trains"
         ButtonGroup.group: radioGroup
         width: parent.width
         property int scenario: 0
      }

      DLRButton {
         text: "Single Slip Coaching"
         checkable: true
         ButtonGroup.group: radioGroup
         width: parent.width
         property int scenario: 1
      }

      DLRButton {
         text: "Double Slip Coaching"
         checkable: true
         ButtonGroup.group: radioGroup
         width: parent.width
         property int scenario: 2
      }
   }
}