import QtQuick 2.15
import QtGraphicalEffects 1.0
import QtQuick.Controls 2.14

import "./components"

Item {
   anchors.fill: parent

   readonly property var touchMap: map

   BusyIndicator {
      anchors.centerIn: parent
      running: overviewImage.status != Image.Ready
      enabled: running
   }

   TouchMap {
      id: map

      mapWidth: root.width
      mapHeight: root.height

      visible: overviewImage.status == Image.Ready

      Image {
         id: "overviewImage"
         anchors.fill: parent
         source: "../../images/germany-grey.webp"
         fillMode: Image.PreserveAspectFit
         cache: true
         mipmap: true
         asynchronous: true
      }

      Item {
         anchors {
            left: overviewImage.left
            right: overviewImage.right
            top: overviewImage.top
            bottom: overviewImage.bottom
            leftMargin: mapHorizontalMargin
            rightMargin: mapHorizontalMargin
            topMargin: mapVerticalMargin
            bottomMargin: mapVerticalMargin
         }

         ListView {
            id: nodeList
            anchors.fill: parent
            model: overviewModel.nodes
            delegate: Node {
               latitude: model.latitude
               longitude: model.longitude
               uid: model.uid
            }
         }

         ListView {
            id: trackList
            anchors.fill: parent
            model: overviewModel.tracks
            delegate: Track {
               xStart: xFromLong(model.startLon)
               yStart: yFromLat(model.startLat)
               xEnd: xFromLong(model.endLon)
               yEnd: yFromLat(model.endLat)
               length: model.length
            }
         }

         ListView {
            id: stationList
            anchors.fill: parent
            model: overviewModel.stations
            delegate: Station {
               latitude: model.latitude
               longitude: model.longitude
               name: model.name
               displayName: model.displayName
               showLeft: model.showLeft
            }
         }

         ListView {
            id: trainList
            anchors.fill: parent
            model: overviewModel.trains
            delegate: Train {
               latitude: model.lat
               longitude: model.lon
               trainId: model.trainId
            }
         }
      }
   }

   property var mapHorizontalMargin: (overviewImage.width - overviewImage.paintedWidth) / 2
   property var mapVerticalMargin: (overviewImage.height - overviewImage.paintedHeight) / 2

   function mercatorProjectLong(longitude) {
      return (longitude + 180) / 360;
   }

   function mercatorProjectLat(latitude) {
      return (1 - Math.log(Math.tan(latitude * Math.PI / 180) + 1 / Math.cos(latitude * Math.PI / 180)) / Math.PI) / 2 * Math.pow(2, 0);
   }

   property var northernBound: mercatorProjectLat(55.0583148)
   property var southernBound: mercatorProjectLat(47.2700983)
   property var  westernBound: mercatorProjectLong(5.8663696)
   property var  easternBound: mercatorProjectLong(15.0415436)
   property var longitudeSize: easternBound - westernBound
   property var  latitudeSize: northernBound - southernBound

   function xFromLong(longitude) {
      return ((mercatorProjectLong(longitude) - westernBound) / longitudeSize) * overviewImage.paintedWidth;
   }

   function yFromLat(latitude) {
      const tmpY = ((mercatorProjectLat(latitude) - southernBound) / latitudeSize) * overviewImage.paintedHeight;
      return overviewImage.paintedHeight - tmpY;
   }
}