import QtQuick 2.15

Flickable {
   id: flick

   default property alias contents: placeholder.children

   property real mapWidth
   property real mapHeight

   readonly property real minZoom: 1
   property real currentZoom: 1
   readonly property real maxZoom: 8

   contentWidth: mapWidth
   contentHeight: mapHeight

   anchors.fill: parent

   PinchArea {
      id: pinchable
      width: Math.max(flick.contentWidth, flick.width)
      height: Math.max(flick.contentHeight, flick.height)

      property real initialWidth
      property real initialHeight

      onPinchStarted: {
         initialWidth = flick.contentWidth
         initialHeight = flick.contentHeight
      }

      onPinchUpdated: {
         function  clamp(num, a, b) { return Math.max(Math.min(num, Math.max(a, b)), Math.min(a, b)) }

         // adjust content pos due to drag
         flick.contentX += pinch.previousCenter.x - pinch.center.x;
         flick.contentY += pinch.previousCenter.y - pinch.center.y;

         //
         const newWidth = clamp(initialWidth * pinch.scale, minZoom * mapWidth, maxZoom * mapWidth);
         const newHeight = clamp(initialHeight * pinch.scale, minZoom * mapHeight, maxZoom * mapHeight);

         // resize content
         flick.resizeContent(newWidth, newHeight, pinch.center);
         flick.currentZoom = contentWidth / mapWidth;
      }

      onPinchFinished: {
         // Move its content within bounds.
         flick.returnToBounds()
      }

      Item {
         id: placeholder
         width: flick.contentWidth
         height: flick.contentHeight
      }

      MouseArea {
         anchors.fill: parent

         onDoubleClicked: {
            flick.contentWidth = mapWidth
            flick.contentHeight = mapHeight
            flick.contentX = 0
            flick.contentY = 0
         }

         onWheel: {
            const startWidth = flick.contentWidth;
            const startHeight = flick.contentHeight;

            let zoomScale = 1;

            if (wheel.angleDelta.y < 0) {
               zoomScale = 0.5;
            } else if (wheel.angleDelta.y > 0) {
               zoomScale = 2;
            }

            function  clamp(num, a, b) { return Math.max(Math.min(num, Math.max(a, b)), Math.min(a, b)) }

            const newWidth = clamp(startWidth * zoomScale, minZoom * mapWidth, maxZoom * mapWidth);
            const newHeight = clamp(startHeight * zoomScale, minZoom * mapHeight, maxZoom * mapHeight);

            // resize content
            flick.resizeContent(newWidth, newHeight, Qt.point(wheel.x, wheel.y));
            flick.returnToBounds();
            flick.currentZoom = contentWidth / mapWidth;
         }

         cursorShape: Qt.CrossCursor
      }
   }
}