import QtQuick 2.14
import QtQuick.Controls 2.14
import QtQuick.Layouts 1.14
import QtQuick.Window 2.15
import QtQuick.Extras 1.4
import QtQuick.Shapes 1.14

import "../components"
import "../theme"
import "simulation-dsc.mjs" as NGT

Item {
   id: view
   anchors.fill: parent

   Component.onCompleted: NGT.createSimulation();

   readonly property real wagonLengthMeter: 21
   readonly property real halfWagonLengthMeter: wagonLengthMeter / 2
   readonly property real imageLengthMeters: 114

   readonly property int imageWidth: Window.width
   readonly property int imageHeight: Window.height

   readonly property real imageScaleW: imageWidth / 3840
   readonly property real imageScaleH: imageHeight / 2160

   Row {
      id: timeRow
      scale: widgetScale
      transformOrigin: Item.TopRight

      anchors {
         top: parent.top
         right: parent.right
         rightMargin: horizontalMargin
         topMargin: verticalMargin + (titleLabel.height * widgetScale) + titleLabel.anchors.topMargin
      }

      z: 100
      width: 0.25 * referenceWidth

      ValueSlider {
         id: speedSlider
         width: parent.width - rewindButton.width

         title: "Simulation Speed"
         unit: "x"

         start: 0
         current: 0
         end: 10
         step: 1
      }

      DLRButton {
         id: rewindButton
         icon.source: "../../icons/rewind.png"
         onClicked: view.restart();
         width: height + 4

         anchors {
            bottom: parent.bottom
            bottomMargin: 6
         }
      }
   }

   Rectangle {
      anchors {
         left: parent.left
         right: parent.right
         top: parent.top
      }
      height: parent.height / 2

      color: "#739535"
   }

   Rectangle {
      anchors {
         left: parent.left
         right: parent.right
         bottom: parent.bottom
      }
      height: parent.height / 2

      color: "#517832"
   }

   Item {
      id: trackMap
      width: NGT.Track.end / imageLengthMeters * imageWidth
      height: imageHeight

      function meterToPixel(meter) {
         return meter * (width / NGT.Track.end);
      }

      function meterToPercent(meter) {
         return meter / NGT.Track.end;
      }

      anchors {
         verticalCenter: parent.verticalCenter
      }

      readonly property real leftBound: Math.min(trackTrainA.back, trackTrainB.x) - 500
      readonly property real rightBound: Math.max(trackTrainA.front, trackTrainD.x + trackTrainD.width) + 500
      readonly property real visibleRegion: rightBound - leftBound

      scale: imageWidth / visibleRegion
      transformOrigin: Item.Left
      x: scale * -leftBound

      component TrackSection : Item {
         property real startMeters
         property real endMeters
         readonly property real lengthMeters: endMeters - startMeters
         x: startMeters / imageLengthMeters * imageWidth
         width: lengthMeters / imageLengthMeters * imageWidth
         height: parent.height
      }

      function getRandomBackground() {
         return "../../images/top-down/E1-Hintergrund-TV-" + Math.floor(Math.random() * 5 + 1) + ".webp"
      }

      Repeater {
         model: NGT.Track.end / imageLengthMeters
         Image {
            x: index * imageWidth
            width: imageWidth
            height: imageHeight
            source: trackMap.getRandomBackground()
            asynchronous: true
            mipmap: true
         }
      }

      TrackSection {
         id: trackStart
         startMeters: NGT.Track.start
         endMeters: NGT.Track.entrySwitch

         Repeater {
            model: trackStart.lengthMeters / imageLengthMeters
            delegate: Image  {
               x: index * imageWidth
               width: imageWidth
               height: imageHeight
               source: "../../images/top-down/E2-Einzelschiene TV.webp"
               asynchronous: true
               mipmap: true
            }
         }
      }

      TrackSection {
         id: trackEntrySwitch
         startMeters: NGT.Track.entrySwitch
         endMeters: NGT.Track.entrySwitch + imageLengthMeters
         Image {
            anchors.fill: parent
            source: "../../images/top-down/E2-Weiche 1zu2-TopView.webp"
            asynchronous: true
            mipmap: true
         }
      }

      TrackSection {
         id: trackBeforeStation
         startMeters: NGT.Track.entrySwitch + imageLengthMeters
         endMeters: NGT.Track.station - 3 * imageLengthMeters

         Repeater {
            model: trackBeforeStation.lengthMeters / imageLengthMeters
            delegate: Image  {
               x: index * imageWidth
               width: imageWidth
               height: imageHeight
               source: "../../images/top-down/E2-Doppelschiene TV.webp"
               asynchronous: true
               mipmap: true
            }
         }
      }

      TrackSection {
         id: trackStation
         startMeters: NGT.Track.station - 3 * imageLengthMeters
         endMeters: NGT.Track.station

         Repeater {
            model: trackStation.lengthMeters / imageLengthMeters
            delegate: Item  {
               x: index * imageWidth
               width: imageWidth
               height: imageHeight
               Image {
                  anchors.fill: parent
                  source: "../../images/top-down/E2-Doppelschiene TV.webp"
                  asynchronous: true
                  mipmap: true
               }
               Image {
                  anchors.fill: parent
                  source: "../../images/top-down/E6-Bahnhof-Doppel.webp"
                  asynchronous: true
                  mipmap: true
               }
            }
         }
      }

      TrackSection {
         id: trackAfterStation
         startMeters: NGT.Track.station
         endMeters: NGT.Track.exitSwitch - imageLengthMeters
         Repeater {
            model: trackAfterStation.lengthMeters / imageLengthMeters
            delegate: Image  {
               x: index * imageWidth
               width: imageWidth
               height: imageHeight
               source: "../../images/top-down/E2-Doppelschiene TV.webp"
               asynchronous: true
               mipmap: true
            }
         }
      }

      TrackSection {
         id: trackExitSwitch
         startMeters: NGT.Track.exitSwitch - imageLengthMeters
         endMeters: NGT.Track.exitSwitch
         Image {
            anchors.fill: parent
            source: "../../images/top-down/E2-Weiche 2zu1-TopView.webp"
            asynchronous: true
            mipmap: true
         }
      }

      TrackSection {
         id: trackEnd
         startMeters: NGT.Track.exitSwitch
         endMeters: NGT.Track.end

         Repeater {
            model: trackEnd.lengthMeters / imageLengthMeters
            delegate: Image  {
               x: index * imageWidth
               width: imageWidth
               height: imageHeight
               source: "../../images/top-down/E2-Einzelschiene TV.webp"
               asynchronous: true
               mipmap: true
            }
         }
      }

      readonly property real upperTrackCenter: 0.485 * imageHeight
      readonly property real lowerTrackCenter: 0.572 * imageHeight

      Path {
         id: trackPath
         startX: 0
         startY: trackMap.upperTrackCenter

         PathLine { x: trackEntrySwitch.x + 0.1 * imageWidth; y: trackMap.upperTrackCenter }
         PathLine { x: trackBeforeStation.x - 0.1 * imageWidth; y: trackMap.lowerTrackCenter }
         PathLine { x: trackExitSwitch.x + 0.1 * imageWidth; y: trackMap.lowerTrackCenter }
         PathLine { x: trackEnd.x - 0.1 * imageWidth; y: trackMap.upperTrackCenter }
         PathLine { x: trackEnd.x + trackEnd.width; y: trackMap.upperTrackCenter }
      }

      component RailCarRight : Item {
         width: 712 * imageScaleW
         height: 108 * imageScaleH
         Image {
            x: -91 * imageScaleW
            y: -24 * imageScaleH
            width: sourceSize.width * imageScaleW
            height: sourceSize.height * imageScaleH
            source: "../../images/top-down/E3-Triebwagen rechts.webp"
            asynchronous: true
            mipmap: true
         }
      }

      component Wagon : Item {
         width: 691 * imageScaleW
         height: 108 * imageScaleH
         Image {
            x: -105 * imageScaleW
            y: -24 * imageScaleH
            width: sourceSize.width * imageScaleW
            height: sourceSize.height * imageScaleH
            source: "../../images/top-down/E4-Mittelwagen.webp"
            asynchronous: true
            mipmap: true
         }
      }

      component RailCarLeft : Item {
         width: 722 * imageScaleW
         height: 108 * imageScaleH
         Image {
            x: -83 * imageScaleW
            y: -24 * imageScaleH
            width: sourceSize.width * imageScaleW
            height: sourceSize.height * imageScaleH
            source: "../../images/top-down/E5-Triebwagen links.webp"
            asynchronous: true
            mipmap: true
         }
      }

      Item {
         id: trackTrainA

         property real trackPositionMeter: 12 * wagonLengthMeter
         readonly property real front: railCarRightA.trackPosition.x
         readonly property real back: railCarLeftA.x

         anchors.fill: parent

         RailCarRight {
            readonly property real trackPositionMeter: trackTrainA.trackPositionMeter - halfWagonLengthMeter
            readonly property var trackPosition: trackPath.pointAtPercent(trackMap.meterToPercent(trackPositionMeter))
            id: railCarRightA
            x: trackPosition.x - width / 2
            y: trackPosition.y - height / 2

            readonly property var frontPoint: trackPath.pointAtPercent((trackPosition.x - width) / parent.width)
            readonly property var backPoint: trackPath.pointAtPercent(trackPosition.x / parent.width)
            readonly property real angle: Math.atan2(backPoint.y - frontPoint.y, backPoint.x - frontPoint.x)

            transform: Rotation {
               origin.x: railCarRightA.width / 2
               origin.y: railCarRightA.height / 2
               angle: railCarRightA.angle * 180 / Math.PI
            }
         }

         Repeater {
            model: 10
            delegate: Wagon {
               id: railWagon
               readonly property real trackPositionMeter: trackTrainA.trackPositionMeter - (wagonLengthMeter - 0.5) * (index + 1) - halfWagonLengthMeter
               readonly property var trackPosition: trackPath.pointAtPercent(trackMap.meterToPercent(trackPositionMeter))
               x: trackPosition.x - width / 2
               y: trackPosition.y - height / 2

               readonly property var frontPoint: trackPath.pointAtPercent((trackPosition.x - width) / parent.width)
               readonly property var backPoint: trackPath.pointAtPercent(trackPosition.x / parent.width)
               readonly property real angle: Math.atan2(backPoint.y - frontPoint.y, backPoint.x - frontPoint.x)

               transform: Rotation {
                  origin.x: railWagon.width / 2
                  origin.y: railWagon.height / 2
                  angle: railWagon.angle * 180 / Math.PI
               }
            }
         }

         RailCarLeft {
            id: railCarLeftA
            readonly property real trackPositionMeter: trackTrainA.trackPositionMeter - 11 * (wagonLengthMeter - 0.5) - halfWagonLengthMeter
            readonly property var trackPosition: trackPath.pointAtPercent(trackMap.meterToPercent(trackPositionMeter))
            x: trackPosition.x - width / 2
            y: trackPosition.y - height / 2

            readonly property var frontPoint: trackPath.pointAtPercent((trackPosition.x - width) / parent.width)
            readonly property var backPoint: trackPath.pointAtPercent(trackPosition.x / parent.width)
            readonly property real angle: Math.atan2(backPoint.y - frontPoint.y, backPoint.x - frontPoint.x)

            transform: Rotation {
               origin.x: railCarLeftA.width / 2
               origin.y: railCarLeftA.height / 2
               angle: railCarLeftA.angle * 180 / Math.PI
            }
         }
      }

      Item {
         id: trackTrainB

         property real trackPositionMeter: 252 + 100 + 84

         x: trackMap.meterToPixel(trackPositionMeter) - width
         y: trackMap.upperTrackCenter - height / 2

         width: childrenRect.width
         height: childrenRect.height

         RailCarRight {
            id: railCarRightB
            x: wagon1B.width + wagon2B.width + railCarLeftB.width
         }

         Wagon {
            id: wagon1B
            x: railCarLeftB.width + wagon2B.width
         }

         Wagon {
            id: wagon2B
            x: railCarLeftB.width
         }

         RailCarLeft {
            id: railCarLeftB
         }
      }

      Item {
         id: trackTrainC

         property real trackPositionMeter: 252 + 200 + 84 + 84

         x: trackMap.meterToPixel(trackPositionMeter) - width
         y: trackMap.upperTrackCenter - height / 2

         width: childrenRect.width
         height: childrenRect.height

         RailCarRight {
            id: railCarRightC
            x: wagon1C.width + wagon2C.width + railCarLeftC.width
         }

         Wagon {
            id: wagon1C
            x: railCarLeftC.width + wagon2C.width
         }

         Wagon {
            id: wagon2C
            x: railCarLeftC.width
         }

         RailCarLeft {
            id: railCarLeftC
         }
      }

      Item {
         id: trackTrainD

         property real trackPositionMeter: 252 + 300 + 84 + 84 + 84

         x: trackMap.meterToPixel(trackPositionMeter) - width
         y: trackMap.upperTrackCenter - height / 2

         width: childrenRect.width
         height: childrenRect.height

         RailCarRight {
            id: railCarRightD
            x: wagon1D.width + wagon2D.width + railCarLeftD.width
         }

         Wagon {
            id: wagon1D
            x: railCarLeftD.width + wagon2D.width
         }

         Wagon {
            id: wagon2D
            x: railCarLeftD.width
         }

         RailCarLeft {
            id: railCarLeftD
         }
      }
   }

   Rectangle {
      id: miniMap
      scale: widgetScale
      transformOrigin: Item.Bottom

      anchors {
         horizontalCenter: parent.horizontalCenter
         bottom: parent.bottom
         bottomMargin: verticalMargin
      }

      width: 0.6 * app.referenceWidth
      height: 0.06 * app.referenceHeight
      radius: 12
      border.width: 2
      border.color: "black"
      color: DLRTheme.backgroundLightGrey

      Shape {
         id: trackShape

         anchors.centerIn: parent
         width: 0.9 * parent.width
         height: 0.9 * parent.height

         function getXFromPercent(xPercent) {
            return xPercent * trackShape.width;
         }

         ShapePath {
            id: longTrainPath
            startX: 0
            startY: 0.5 * trackShape.height - 6

            strokeWidth: 2
            fillColor: "transparent"
            strokeColor: "black"
            capStyle: ShapePath.RoundCap

            PathLine {
               property var xPercent: NGT.relativePosition(NGT.Track.entrySwitch);
               x: trackShape.getXFromPercent(xPercent); y: longTrainPath.startY
            }
            PathLine {
               property var xPercent: NGT.relativePosition(NGT.Track.entrySwitch + imageLengthMeters);
               x: trackShape.getXFromPercent(xPercent); y: longTrainPath.startY + 12
            }
            PathLine {
               property var xPercent: NGT.relativePosition(NGT.Track.station);
               x: trackShape.getXFromPercent(xPercent); y: longTrainPath.startY + 12
            }
            PathLine {
               property var xPercent: NGT.relativePosition(NGT.Track.exitSwitch - imageLengthMeters);
               x: trackShape.getXFromPercent(xPercent); y: longTrainPath.startY + 12
            }
            PathLine {
               property var xPercent: NGT.relativePosition(NGT.Track.exitSwitch);
               x: trackShape.getXFromPercent(xPercent); y: longTrainPath.startY
            }
            PathLine {
               property var xPercent: NGT.relativePosition(NGT.Track.end);
               x: trackShape.getXFromPercent(xPercent); y: longTrainPath.startY
            }
         }

         ShapePath {
            id: shortTrainPath
            startX: 0
            startY: 0.5 * trackShape.height - 6

            strokeWidth: 2
            fillColor: "transparent"
            strokeColor: "black"
            capStyle: ShapePath.RoundCap

            PathLine {
               property var xPercent: NGT.relativePosition(NGT.Track.end);
               x: trackShape.getXFromPercent(xPercent); y: shortTrainPath.startY
            }
         }
      }

      Item {
         anchors.centerIn: parent
         width: parent.width
         height: 0.9 * parent.height

         Rectangle {
            id: trainA
            height: 7
            radius: 3
            width: trackShape.getXFromPercent(NGT.relativePosition(252))
            color: DLRTheme.teaserYellow
            border.color: "black"

            DLRLabel {
               anchors {
                  horizontalCenter: parent.horizontalCenter
                  top: parent.bottom
               }

               text: "U1"
               font.pointSize: 8
            }
         }

         Rectangle {
            id: trainB
            height: 7
            radius: 3
            width: trackShape.getXFromPercent(NGT.relativePosition(84))
            color: DLRTheme.teaserYellow
            border.color: "black"

            DLRLabel {
               anchors {
                  horizontalCenter: parent.horizontalCenter
                  bottom: parent.top
               }

               text: "4"
               font.pointSize: 8
            }
         }

         Rectangle {
            id: trainC
            height: 7
            radius: 3
            width: trackShape.getXFromPercent(NGT.relativePosition(84))
            color: DLRTheme.teaserYellow
            border.color: "black"

            DLRLabel {
               anchors {
                  horizontalCenter: parent.horizontalCenter
                  bottom: parent.top
               }

               text: "3"
               font.pointSize: 8
            }
         }

         Rectangle {
            id: trainD
            height: 7
            radius: 3
            width: trackShape.getXFromPercent(NGT.relativePosition(84))
            color: DLRTheme.teaserYellow
            border.color: "black"

            DLRLabel {
               anchors {
                  horizontalCenter: parent.horizontalCenter
                  bottom: parent.top
               }

               text: "2"
               font.pointSize: 8
            }
         }

         Rectangle {
            id: station
            width: trackShape.getXFromPercent(NGT.relativePosition(300))
            height: 7
            border.color: "black"
            color: DLRTheme.teaserGreen

            DLRLabel {
               anchors {
                  verticalCenter: parent.verticalCenter
                  right: parent.left
                  rightMargin: 2
               }

               text: "Station"
               font.pointSize: 7
            }
         }
      }
   }



   function render() {
      // trains
      trackTrainA.trackPositionMeter = NGT.simulation.trainA.position;
      trackTrainB.trackPositionMeter = NGT.simulation.trainB.position;
      trackTrainC.trackPositionMeter = NGT.simulation.trainC.position;
      trackTrainD.trackPositionMeter = NGT.simulation.trainD.position;

      infoText.text = NGT.simulation.infoText;

      // minimap
      {
         const trainAPos = NGT.relativePosition(NGT.simulation.trainA.position - 126);
         const p = longTrainPath.pointAtPercent(trainAPos);
         trainA.x = trackShape.x + p.x - (trainA.width / 2);
         trainA.y = p.y - trainA.height / 2;
      }

      {
         const trainBPos = NGT.relativePosition(NGT.simulation.trainB.position);
         const p = shortTrainPath.pointAtPercent(trainBPos);
         trainB.x = trackShape.x + p.x - trainB.width;
         trainB.y = p.y - trainB.height / 2;
      }

      {
         const trainCPos = NGT.relativePosition(NGT.simulation.trainC.position);
         const p = shortTrainPath.pointAtPercent(trainCPos);
         trainC.x = trackShape.x + p.x - trainC.width;
         trainC.y = p.y - trainC.height / 2;
      }

      {
         const trainDPos = NGT.relativePosition(NGT.simulation.trainD.position);
         const p = shortTrainPath.pointAtPercent(trainDPos);
         trainD.x = trackShape.x + p.x - trainD.width;
         trainD.y = p.y - trainD.height / 2;
      }

      {
         const stationPos = NGT.relativePosition(NGT.Track.station);
         const p = shortTrainPath.pointAtPercent(stationPos);
         station.x = trackShape.x + p.x - station.width;
         station.y = p.y + 2.5
      }
   }

   Rectangle {
      scale: widgetScale
      transformOrigin: Item.Center
      z: 100
      anchors {
         centerIn: parent
         verticalCenterOffset: -0.15 * imageHeight
      }
      width: 500
      height: infoText.height + 20
      radius: 10
      border.width: 2
      border.color: "black"
      color: DLRTheme.backgroundLightGrey

      DLRLabel {
         id: infoText
         anchors {
            centerIn: parent
         }
         width: parent.width - 20
         horizontalAlignment: Text.AlignHCenter
         verticalAlignment: Text.AlignVCenter

         wrapMode: Text.Wrap
         font.pointSize: 14
      }
   }

   function restart() {
      NGT.createSimulation();
      simLoop.wait = 0;
   }

   Timer {
      id: simLoop
      interval: 12
      running: true
      repeat: true

      property real wait: 0

      onTriggered: {
         if (NGT.simulation.finished) {
            if (wait < 3000) {
               wait += interval;
            } else {
               NGT.createSimulation();
               wait = 0;
            }
         } else {
            const elapsed = (interval / 1000) * speedSlider.current;
            NGT.simulation.simulate(elapsed);
            render();
            //timeDisplay.text = NGT.toMinutesTime(NGT.simulation.elapsedTime);
         }
      }
   }

   DLRButton {
      anchors {
         bottom: parent.bottom
         right: parent.right
         rightMargin: horizontalMargin
         bottomMargin: 1.5 * verticalMargin + overviewButton.height * widgetScale
      }

      scale: widgetScale
      transformOrigin: Item.BottomRight

      text: "Show Info"
      onClicked: scenarioDescription.open()
   }

   ScenarioDescription {
      id: scenarioDescription
      z: 1000

      title: "Double Slip Coaching"
      descriptionURL: ":/gui/slip-coaching/dsc-description.html"

      onOpened: speedSlider.current = 0
      onClosed: speedSlider.current = 1
   }

   ReturnToOverviewButton {
      id: overviewButton
      z: 1001
      anchors {
         bottom: parent.bottom
         right: parent.right
         rightMargin: horizontalMargin
         bottomMargin: verticalMargin
      }
   }
}