#pragma once

#include <glm/ext/scalar_constants.hpp>
#include <glm/gtx/vector_angle.hpp>
#include <glm/vec2.hpp>
#include <iostream>

namespace ngt {
template <typename T>
constexpr auto toRadians(glm::tvec2<T> const& coordinates) -> glm::tvec2<T> {
    return coordinates / 180.0 * glm::pi<T>();
}

template <typename T>
constexpr auto toDegrees(glm::tvec2<T> const& coordinates) -> glm::tvec2<T> {
    return coordinates / glm::pi<T>() * 180.0;
}

template <typename T>
constexpr auto lerpRad(glm::tvec2<T> const& p1, glm::tvec2<T> const& p2, T f) -> glm::tvec2<T> {
    using namespace glm; // NOLINT
    auto delta = angle(p1, p2);
    auto a     = sin((1.0 - f) * delta) / sin(delta);
    auto b     = sin(f * delta) / sin(delta);
    auto x     = a * cos(p1.x) * cos(p1.y) + b * cos(p2.x) * cos(p2.y);
    auto y     = a * cos(p1.x) * sin(p1.y) + b * cos(p2.x) * sin(p2.y);
    auto z     = a * sin(p1.x) + b * sin(p2.x);
    return {atan2(z, sqrt(x * x + y * y)), atan2(y, x)};
}

template <typename T>
constexpr auto lerpDeg(glm::tvec2<T> const& p1, glm::tvec2<T> const& p2, T f) -> glm::tvec2<T> {
    return std::move(toDegrees(lerpRad(toRadians(p1), toRadians(p2), f)));
}
} // namespace ngt