#include "Overview.hpp"

#include <QtQuick/QtQuick>

namespace ngt::overview {

Overview::Overview() {
   stationsModel = std::make_unique<StationListViewModel>(simulator.getStations());
   nodesModel    = std::make_unique<TrackNodeListViewModel>(simulator.getNodes());
   tracksModel   = std::make_unique<TrackSegmentListViewModel>(simulator.getTracks());
   trainsModel   = std::make_unique<TrainListViewModel>(simulator.getTrains());

   connect(this, &Overview::setSpeed, &simulator, &Simulator::setSpeed);
   connect(&simulator, &Simulator::simulationUpdate, this, &Overview::simulationUpdate);

   startSimulation();
}

Overview::~Overview() {
   stopSimulation();
}

void Overview::startSimulation() {
   simulator.startSimulation();
}

void Overview::stopSimulation() {
   simulator.stopSimulation();
}

void Overview::simulationUpdate(int t) {
   if (t != lastTime) {
      pSimulationTime = t;
      lastTime        = t;
   }

   trainsModel->changeEvent();
   emit update();
}

void Overview::setScenario(int scenario) {
   stopSimulation();

   disconnect(this, &Overview::setSpeed, &simulator, &Simulator::setSpeed);
   disconnect(&simulator, &Simulator::simulationUpdate, this, &Overview::simulationUpdate);

   simulator = Simulator(static_cast<uint8_t>(scenario));

   connect(this, &Overview::setSpeed, &simulator, &Simulator::setSpeed);
   connect(&simulator, &Simulator::simulationUpdate, this, &Overview::simulationUpdate);

   startSimulation();
}

void Overview::simulate(int deltaT) {
   simulator.nextTick(std::chrono::milliseconds(deltaT));
}

void Overview::setSimulationSpeed(double newSpeed) {
   if (pSimulationSpeed != newSpeed) {
      pSimulationSpeed = newSpeed;
      emit setSpeed(newSpeed);
   }
}

} // namespace ngt::overview
