#pragma once

#include <memory>

#include <QtQml/QtQml>
#include <ngt/views/overview/control/Simulator.hpp>
#include <ngt/views/overview/model/Train.hpp>
#include <ngt/views/overview/view-model/StationListViewModel.hpp>
#include <ngt/views/overview/view-model/TrackNodeListViewModel.hpp>
#include <ngt/views/overview/view-model/TrackSegmentListViewModel.hpp>
#include <ngt/views/overview/view-model/TrainListViewModel.hpp>

namespace ngt::overview {

const uint8_t STATE_OF_THE_ART_SCENARIO = 0;
const uint8_t SINGLE_SLIP_COACHING_SCENARIO = 1;
const uint8_t DOUBLE_SLIP_COACHING_SCENARIO = 2;

class Overview : public QObject {
   Q_OBJECT; // NOLINT

   Q_PROPERTY(StationListViewModel* stations READ stations CONSTANT);
   Q_PROPERTY(TrackNodeListViewModel* nodes READ nodes CONSTANT);
   Q_PROPERTY(TrackSegmentListViewModel* tracks READ tracks CONSTANT);
   Q_PROPERTY(TrainListViewModel* trains READ trains NOTIFY update);

   Q_PROPERTY(int simulationTime READ simulationTime NOTIFY update);
   Q_PROPERTY(double simulationSpeed READ simulationSpeed WRITE setSimulationSpeed);

 public:
   Overview();

   Overview(Overview const& other) = delete;
   auto operator=(Overview const& other) -> Overview& = delete;

   Overview(Overview&& other) = delete;
   auto operator=(Overview&& other) -> Overview& = delete;

   ~Overview() override;

   void startSimulation();
   void stopSimulation();

   [[nodiscard]]
   auto stations() const -> StationListViewModel* { return stationsModel.get(); }

   [[nodiscard]]
   auto nodes() const -> TrackNodeListViewModel* { return nodesModel.get(); }

   [[nodiscard]]
   auto tracks() const -> TrackSegmentListViewModel* { return tracksModel.get(); }

   [[nodiscard]]
   auto trains() const -> TrainListViewModel* { return trainsModel.get(); }

   [[nodiscard]]
   auto simulationTime() const -> int { return pSimulationTime; }

   [[nodiscard]]
   auto simulationSpeed() const -> double { return pSimulationSpeed; }

   Q_INVOKABLE void setScenario(int scenario);

   Q_INVOKABLE void simulate(int deltaT);

   void setSimulationSpeed(double newSpeed);

 signals:
   void update();

 private:
   std::unique_ptr<StationListViewModel>      stationsModel;
   std::unique_ptr<TrackNodeListViewModel>    nodesModel;
   std::unique_ptr<TrackSegmentListViewModel> tracksModel;
   std::unique_ptr<TrainListViewModel>        trainsModel;

   Simulator simulator{STATE_OF_THE_ART_SCENARIO};

   int pSimulationTime = 0;
   int lastTime        = 0;

   double pSimulationSpeed = 1.0;

 public slots:
   void simulationUpdate(int t);

 signals:
   void setSpeed(double _t1);
};

} // namespace ngt::overview