#include "JsonModels.hpp"

#include <nlohmann/json.hpp>

#include <cstdint>
#include <string>

namespace ngt::overview::json {

void from_json(nlohmann::json const& j, Train& t) {
   t.id      = j.at("train_id").get<std::string>();
   t.waggons = j.at("waggons").get<uint32_t>();
}

void from_json(nlohmann::json const& j, Node& n) {
   n.name        = j.at("name").get<std::string>();
   n.coordinates = j.at("p").get<std::pair<double, double>>();
   n.isStation   = j.at("is_station").get<bool>();

   if (n.isStation) {
      n.displayName = j.at("display_name").get<bool>();
   }

   if (n.displayName) {
      n.shortName = j.at("short_name").get<std::string>();
      n.left      = j.at("left").get<bool>();
   }
}

void from_json(nlohmann::json const& j, Edge& e) {
   auto idField = j.at("edge_id");
   if (idField.is_number_integer()) {
      e.id = std::to_string(idField.get<uint32_t>());
   } else {
      e.id = idField.get<std::string>();
   }

   e.length = j.at("length").get<double>();
   e.start  = j.at("node0").get<std::string>();
   e.end    = j.at("node1").get<std::string>();
}

void from_json(nlohmann::json const& j, TimeStamp& t) {
   t.time = j.at(0).get<double>() - 1000.0;

   auto idField = j.at(1);
   if (idField.is_number_integer()) {
      t.edge = std::to_string(idField.get<uint32_t>());
   } else {
      t.edge = idField.get<std::string>();
   }

   t.edgePosition = j.at(2).get<double>();
}

void from_json(nlohmann::json const& j, std::map<std::string, TrainPositions>& t) {
   for (auto const& [key, value] : j.items()) {
      if (!value.empty()) {
         t[key] = {key, value.get<std::vector<TimeStamp>>()};
      }
   }
}

}; // namespace ngt::overview::json