#pragma once

#include <map>
#include <memory>
#include <vector>

#include <QtCore/QObject>

#include <ngt/views/overview/model/Station.hpp>
#include <ngt/views/overview/model/TrackNode.hpp>
#include <ngt/views/overview/model/TrackSegment.hpp>
#include <ngt/views/overview/model/Train.hpp>

namespace ngt::overview {

class Simulator : public QObject {
   Q_OBJECT; // NOLINT
 public:
   explicit Simulator(uint8_t scenario);

   Simulator(Simulator const& other);
   Simulator(Simulator&& other) noexcept;

   auto operator=(Simulator const& other) -> Simulator&;
   auto operator=(Simulator&& other) noexcept -> Simulator&;

   ~Simulator() override = default;

   void startSimulation();
   void nextTick(std::chrono::milliseconds deltaT);
   void simulate(std::chrono::milliseconds t);
   void stopSimulation();

   [[nodiscard]]
   auto getTrains() const -> std::vector<Train>* { return trains; }

   [[nodiscard]]
   auto getStations() const -> std::vector<Station> const* { return stations; }

   [[nodiscard]]
   auto getNodes() const -> std::unordered_map<std::string, TrackNode> const* { return trackNodes; }

   [[nodiscard]]
   auto getTracks() const -> std::unordered_map<std::string, TrackSegment> const* {
      return trackSegments;
   }

 private:
   [[nodiscard]]
   auto coordinatesFromTrackPosition(std::string_view trackId, double position) const noexcept -> Coordinates;

   std::vector<Train>*                            trains;
   std::vector<Station>*                          stations;
   std::unordered_map<std::string, TrackNode>*    trackNodes;
   std::unordered_map<std::string, TrackSegment>* trackSegments;

   std::unordered_map<std::string, TrainData>* trainData;

   std::chrono::milliseconds maxTime{};
   std::chrono::milliseconds simulationTime{};

   double  speed{1.0};
   uint8_t scenario;

 public slots:
   void setSpeed(double newSpeed);

 signals:
   void simulationUpdate(int _t1);
};

} // namespace ngt::overview