#pragma once

#include "TrainData.hpp"

#include <ngt/utils/Coordinates.hpp>
#include <ngt/utils/si.hpp>

#include <string>
#include <utility>

namespace ngt::overview {

static constexpr double const WAGON_LENGTH = 0.005;
static constexpr double const WAGON_GAP    = 0.001;

struct Wagon {
   Wagon() = default;

   uint32_t positionInTrain{0};

   /// The center of the wagon.
   Coordinates coordinates{0, 0};

   /// The direction the wagon is driving.
   si::degree_t orientation{0};
};

struct Train {
   Train(std::string id, uint32_t numWagons)
      : id(std::move(id))
      , wagons(numWagons) {
      for (uint32_t i = 0; i < numWagons; ++i) {
         wagons[i].positionInTrain = i;
      }
   }

   /// Coordinates of the front of the first wagon.
   Coordinates coordinates{0, 0};

   std::string        id;
   std::vector<Wagon> wagons;
};

} // namespace ngt::overview