#include "TrainData.hpp"

namespace ngt::overview {

TrainData::TrainData(std::string_view id, std::map<std::chrono::milliseconds, TimeStepData> timeSteps)
   : id(id)
   , endTime(timeSteps.rbegin()->first)
   , timeSteps(std::move(timeSteps)) {
   using namespace std::chrono_literals;

   std::vector<std::chrono::milliseconds> toDelete{};

   auto        prev         = this->timeSteps.begin();
   std::string prevTrack    = prev->second.trackId;
   double      prevPosition = prev->second.position;

   for (auto const& [time, current] : this->timeSteps) {
      if (time == 0ms) {
         continue;
      }

      if (current.trackId == prevTrack && current.position == prevPosition) {
         toDelete.push_back(time);
      } else {
         prevTrack    = current.trackId;
         prevPosition = current.position;
      }
   }

   for (std::chrono::milliseconds t : toDelete) {
      this->timeSteps.erase(t);
   }
}

auto TrainData::getPrevTimeStep(std::chrono::milliseconds t) const
   -> std::optional<std::reference_wrapper<TimeStepData const>> {
   auto it = getTimeStepIt(t);
   if (it == timeSteps.begin()) {
      return std::nullopt;
   }

   return std::prev(it)->second;
}

auto TrainData::getTimeStep(std::chrono::milliseconds t) const -> TimeStepData const& {
   return getTimeStepIt(t)->second;
}

auto TrainData::getNextTimeStep(std::chrono::milliseconds t) const
   -> std::optional<std::reference_wrapper<TimeStepData const>> {
   auto it = getTimeStepIt(t);
   if (std::next(it) == timeSteps.end()) {
      return std::nullopt;
   }

   return std::next(it)->second;
}

auto TrainData::findPreviousEdge(std::chrono::milliseconds t) const -> std::optional<TimeStepData const> {
   auto it = getTimeStepIt(t);

   std::string trackId = it->second.trackId;
   while (it != timeSteps.begin() && it->second.trackId == trackId) {
      it = std::prev(it);
   }

   if (it->second.trackId != trackId) {
      return it->second;
   }

   return std::nullopt;
}

} // namespace ngt::overview