#pragma once

#include <chrono>
#include <iostream>
#include <optional>
#include <string>
#include <utility>
#include <vector>

#include <ngt/utils/io.hpp>

namespace ngt::overview {

struct TimeStepData {
   TimeStepData() = default;

   TimeStepData(std::chrono::milliseconds timeStep, std::string_view trackId, double position)
      : timeStep(timeStep)
      , trackId(trackId)
      , position(position) {}

   std::chrono::milliseconds timeStep{};
   std::string               trackId{};
   double                    position{};

   friend auto operator<<(std::ostream& os, TimeStepData const& data) -> std::ostream& {
      os << "timeStep: " << data.timeStep.count() << " trackId: " << data.trackId << " position: " << data.position;
      return os;
   }
};

struct TrainData {
   TrainData(std::string_view id, std::map<std::chrono::milliseconds, TimeStepData> timeSteps);

   std::string id;

   [[nodiscard]]
   auto getPrevTimeStep(std::chrono::milliseconds t) const -> std::optional<std::reference_wrapper<TimeStepData const>>;

   [[nodiscard]]
   auto getNextTimeStep(std::chrono::milliseconds t) const -> std::optional<std::reference_wrapper<TimeStepData const>>;

   [[nodiscard]]
   auto getTimeStep(std::chrono::milliseconds t) const -> TimeStepData const&;

   [[nodiscard]]
   auto findPreviousEdge(std::chrono::milliseconds t) const -> std::optional<TimeStepData const>;

   friend auto operator<<(std::ostream& os, TrainData const& data) -> std::ostream& {
      os << "id: " << data.id << " timeSteps: " << data.timeSteps.size();
      return os;
   }

 private:
   std::chrono::milliseconds                         endTime;
   std::map<std::chrono::milliseconds, TimeStepData> timeSteps;

   [[nodiscard]]
   auto getTimeStepIt(std::chrono::milliseconds t) const {
      if (t > endTime) {
         return std::prev(timeSteps.end());
      }

      auto it = timeSteps.lower_bound(t);
      if (it == timeSteps.begin()) {
         return it;
      }
      return std::prev(it);
   }
};

} // namespace ngt::overview
