#include "StationListViewModel.hpp"

#include <ngt/views/overview/model/Station.hpp>
#include <utility>

namespace ngt::overview {

StationListViewModel::StationListViewModel(std::vector<Station> const* const stations)
   : QAbstractListModel(nullptr)
   , stations(stations) {}

auto StationListViewModel::rowCount(QModelIndex const& parent) const -> int {
   if (parent.isValid()) {
      return 0;
   }

   return static_cast<int32_t>(stations->size());
}

auto StationListViewModel::roleNames() const -> QHash<int, QByteArray> {
   QHash<int, QByteArray> fields;
   fields[Latitude]    = "latitude";
   fields[Longitude]   = "longitude";
   fields[Name]        = "name";
   fields[ShortName]   = "shortName";
   fields[DisplayName] = "displayName";
   fields[Left]        = "showLeft";
   return fields;
}

auto StationListViewModel::data(const QModelIndex& index, int role) const -> QVariant {
   if (!index.isValid()) {
      return QVariant();
   }

   const int      i       = index.row();
   Station const& station = stations->at(i);

   switch (role) {
      case Latitude:
         return QVariant(station.coordinates.latitude);
      case Longitude:
         return QVariant(station.coordinates.longitude);
      case Name:
         return QVariant(station.name.data());
      case ShortName:
         return QVariant(station.shortName.data());
      case DisplayName:
         return QVariant(station.displayName);
      case Left:
         return QVariant(station.left);
      default:
         return QVariant();
   }
}

auto StationListViewModel::operator=(const StationListViewModel& other) -> StationListViewModel& {
   if (this != &other) {
      this->stations = other.stations;
   }

   return *this;
}

} // namespace ngt::overview