#include "TrackNodeListViewModel.hpp"

#include <utility>

namespace ngt::overview {

TrackNodeListViewModel::TrackNodeListViewModel(std::unordered_map<std::string, TrackNode> const* const trackNodes)
   : QAbstractListModel(nullptr)
   , trackNodes(trackNodes) {}

auto TrackNodeListViewModel::rowCount(QModelIndex const& parent) const -> int {
   if (parent.isValid()) {
      return 0;
   }

   return static_cast<int32_t>(trackNodes->size());
}

auto TrackNodeListViewModel::roleNames() const -> QHash<int, QByteArray> {
   QHash<int, QByteArray> fields;
   fields[Latitude]  = "latitude";
   fields[Longitude] = "longitude";
   fields[Uid]       = "uid";
   return fields;
}

auto TrackNodeListViewModel::data(const QModelIndex& index, int role) const -> QVariant {
   if (!index.isValid()) {
      return QVariant();
   }

   const size_t     i         = index.row();
   TrackNode const& trackNode = std::next(trackNodes->begin(), i)->second;

   switch (role) {
      case Latitude:
         return QVariant(trackNode.coordinates.latitude);
      case Longitude:
         return QVariant(trackNode.coordinates.longitude);
      case Uid:
         return QVariant(trackNode.id.data());
      default:
         return QVariant();
   }
}

auto TrackNodeListViewModel::operator=(const TrackNodeListViewModel& other) -> TrackNodeListViewModel& {
   if (this != &other) {
      this->trackNodes = other.trackNodes;
   }

   return *this;
}

} // namespace ngt::overview