#pragma once

#include <QtCore/QAbstractListModel>
#include <ngt/views/overview/model/TrackNode.hpp>
#include <vector>

namespace ngt::overview {

class TrackNodeListViewModel : public QAbstractListModel {
   Q_OBJECT; // NOLINT

 public:
   explicit TrackNodeListViewModel(std::unordered_map<std::string, TrackNode> const* trackNodes = nullptr);

   TrackNodeListViewModel(TrackNodeListViewModel const& other) = delete;
   auto operator=(TrackNodeListViewModel const& other) -> TrackNodeListViewModel&;

   TrackNodeListViewModel(TrackNodeListViewModel&& other) = delete;
   auto operator=(TrackNodeListViewModel&& other) -> TrackNodeListViewModel& = delete;

   ~TrackNodeListViewModel() override = default;

   [[nodiscard]]
   auto rowCount(QModelIndex const& parent) const -> int override;

   [[nodiscard]]
   auto data(const QModelIndex& index, int role) const -> QVariant override;

   [[nodiscard]]
   auto roleNames() const -> QHash<int, QByteArray> override;

   [[nodiscard]]
   auto getNode(std::string_view id) const -> TrackNode const& { return trackNodes->at(id.data()); }

 private:
   std::unordered_map<std::string, TrackNode> const* trackNodes;

   enum { Latitude = Qt::UserRole, Longitude, Uid };
};

} // namespace ngt::overview