#include "TrackSegmentListViewModel.hpp"

namespace ngt::overview {

enum { StartLat = Qt::UserRole, StartLon, EndLat, EndLon, Length };

TrackSegmentListViewModel::TrackSegmentListViewModel(
   std::unordered_map<std::string, TrackSegment> const* const trackSegments)
   : QAbstractListModel(nullptr)
   , trackSegments(trackSegments) {}

auto TrackSegmentListViewModel::rowCount(QModelIndex const& parent) const -> int {
   if (parent.isValid()) {
      return 0;
   }

   return static_cast<int32_t>(trackSegments->size());
}

auto TrackSegmentListViewModel::roleNames() const -> QHash<int, QByteArray> {
   QHash<int, QByteArray> fields;
   fields[StartLat] = "startLat";
   fields[StartLon] = "startLon";
   fields[EndLat]   = "endLat";
   fields[EndLon]   = "endLon";
   fields[Length]   = "length";
   return fields;
}

auto TrackSegmentListViewModel::data(const QModelIndex& index, int role) const -> QVariant {
   if (!index.isValid()) {
      return QVariant();
   }

   const int i  = index.row();
   auto      it = trackSegments->begin();
   std::advance(it, i);
   TrackSegment const& trackSegment = it->second;

   switch (role) {
      case StartLat:
         return QVariant(trackSegment.start.coordinates.latitude);
      case StartLon:
         return QVariant(trackSegment.start.coordinates.longitude);
      case EndLat:
         return QVariant(trackSegment.end.coordinates.latitude);
      case EndLon:
         return QVariant(trackSegment.end.coordinates.longitude);
      case Length:
         return QVariant(trackSegment.length);
      default:
         return QVariant();
   }
}

auto TrackSegmentListViewModel::operator=(const TrackSegmentListViewModel& other) -> TrackSegmentListViewModel& {
   if (this != &other) {
      this->trackSegments = other.trackSegments;
   }

   return *this;
}

} // namespace ngt::overview