#include "TrainListViewModel.hpp"
#include <thread>

namespace ngt::overview {

TrainListViewModel::TrainListViewModel(std::vector<Train>* const trains)
   : QAbstractListModel(nullptr)
   , trains(trains) {}

auto TrainListViewModel::rowCount(QModelIndex const& parent) const -> int {
   if (parent.isValid()) {
      {
         { return 0; }
      }
   }

   size_t count = 0;
   for (auto const& train : *trains) {
      count += train.wagons.size();
   }

   return static_cast<int32_t>(count);
}

enum { Id = Qt::UserRole, Lat, Lon };

auto TrainListViewModel::roleNames() const -> QHash<int, QByteArray> {
   QHash<int, QByteArray> fields;
   fields[Id]  = "trainId";
   fields[Lat] = "lat";
   fields[Lon] = "lon";
   return fields;
}

auto TrainListViewModel::data(QModelIndex const& index, int role) const -> QVariant {
   if (!index.isValid()) {
      {
         { return QVariant(); }
      }
   }

   size_t const i = index.row();

   size_t wagonCounter = 0;
   size_t trainIndex   = 0;
   while (wagonCounter + trains->at(trainIndex).wagons.size() <= i) {
      wagonCounter += trains->at(trainIndex).wagons.size();
      trainIndex++;
   }

   size_t       wagonIndex = i - wagonCounter;
   Train const& train      = trains->at(trainIndex);
   Wagon const& wagon      = train.wagons.at(wagonIndex);

   switch (role) {
      case Id:
         return QVariant(train.id.data());
      case Lat:
         return QVariant(wagon.coordinates.latitude);
      case Lon:
         return QVariant(wagon.coordinates.longitude);
      default:
         return QVariant();
   }
}

void TrainListViewModel::changeEvent() {
   size_t count = 0;
   for (auto const& train : *trains) {
      count += train.wagons.size();
   }

   dataChanged(createIndex(0, 0), createIndex(static_cast<int32_t>(count), 0));
}

auto TrainListViewModel::operator=(const TrainListViewModel& other) -> TrainListViewModel& {
   if (this != &other) {
      this->trains = other.trains;
   }

   return *this;
}

} // namespace ngt::overview
