#include "JsonLoader.hpp"

namespace ngt::speedlimits {

auto loadFromJson(std::string_view file) -> Settings {
   static std::unordered_map<std::string, Settings> cache{};
   if (cache.find(file.data()) != cache.end()) {
      return cache.at(file.data());
   }

   nlohmann::json j = io::loadJson(file);

   auto     sections = j.at("track").get<std::vector<TrackSection>>();
   Settings settings{j.at("trainProperties").get<TrainProperties>(), Track{sections}};

   cache.emplace(file, settings);
   return settings;
}

void from_json(const nlohmann::json& j, TrainProperties& trainProperties) {
   trainProperties.wagons          = j.at("wagons").get<uint8_t>();
   trainProperties.vctsMinDistance = si::parse<si::meter_t>(j, "vctsMinDistance");

   trainProperties.length = si::parse<si::meter_t>(j, "length");

   trainProperties.acceleration = si::parse<si::meters_per_second_squared_t>(j, "acceleration");
   trainProperties.deceleration = si::parse<si::meters_per_second_squared_t>(j, "deceleration");
}

void from_json(const nlohmann::json& j, TrackSection& section) {
   section.length     = si::parse<si::kilometer_t>(j, "length");
   section.speedLimit = si::parse<si::kilometers_per_hour_t>(j, "speedLimit");
   section.min        = si::parse<si::kilometers_per_hour_t>(j, "min");
   section.max        = si::parse<si::kilometers_per_hour_t>(j, "max");
}

} // namespace ngt::speedlimits