#include "Track.hpp"

#include <spdlog/spdlog.h>

namespace ngt::speedlimits {

auto TrackSection::toString() const noexcept -> std::string {
   try {
      return fmt::format("length: {}; speedLimit: {}", si::to_string(length), si::to_string(speedLimit));
   } catch (...) { return ""; }
}

Track::Track(const std::vector<TrackSection>& sections)
   : totalLength(0_km) {
   for (const auto& section : sections) {
      this->sections.try_emplace(totalLength, section);
      totalLength += section.length;
   }
}

auto Track::getSectionAt(si::kilometer_t position) const noexcept -> TrackSection const& {
   auto it = sections.upper_bound(position);
   if (it == sections.begin()) {
      return it->second;
   }

   return std::prev(it)->second;
}

auto Track::getNextSection(si::kilometer_t position) const noexcept -> std::pair<si::kilometer_t, TrackSection> {
   auto const it = sections.upper_bound(position);
   if (it == sections.end()) {
      return *std::prev(it);
   }

   return *it;
}

auto Track::getSections() noexcept -> std::map<si::kilometer_t, TrackSection>& {
   return sections;
}

} // namespace ngt::speedlimits