#pragma once

#include <map>
#include <ngt/utils/si.hpp>

namespace ngt::speedlimits {

struct TrackSection {
   si::kilometer_t           length;
   si::kilometers_per_hour_t speedLimit;
   si::kilometers_per_hour_t min;
   si::kilometers_per_hour_t max;

   [[nodiscard]]
   auto toString() const noexcept -> std::string;
};

class Track {
 public:
   Track() = default;

   explicit Track(std::vector<TrackSection> const& sections);

   [[nodiscard]]
   auto getSectionAt(si::kilometer_t position) const noexcept -> TrackSection const&;

   [[nodiscard]]
   auto getNextSection(si::kilometer_t position) const noexcept -> std::pair<si::kilometer_t, TrackSection>;

   [[nodiscard]]
   auto length() const noexcept -> si::kilometer_t { return totalLength; }

   [[nodiscard]]
   auto getSections() noexcept -> std::map<si::kilometer_t, TrackSection>&;

 private:
   si::kilometer_t                         totalLength = 0_km;
   std::map<si::kilometer_t, TrackSection> sections;
};

} // namespace ngt::speedlimits