#include "TrackSegmentListViewModel.hpp"

namespace ngt::speedlimits {

TrackSegmentListViewModel::TrackSegmentListViewModel(Track* track)
   : QAbstractListModel(nullptr)
   , track(track) {}

auto TrackSegmentListViewModel::operator=(const TrackSegmentListViewModel& other) -> TrackSegmentListViewModel& {
   if (this != &other) {
      track = other.track;
   }

   return *this;
}

auto TrackSegmentListViewModel::operator=(TrackSegmentListViewModel&& other) noexcept -> TrackSegmentListViewModel& {
   if (this != &other) {
      track = other.track;
   }

   return *this;
}

auto TrackSegmentListViewModel::rowCount(const QModelIndex& /*parent*/) const -> int {
   return static_cast<int32_t>(track->getSections().size());
}

auto TrackSegmentListViewModel::roleNames() const -> QHash<int, QByteArray> {
   QHash<int, QByteArray> fields;
   fields[Start]      = "start";
   fields[Length]     = "length";
   fields[SpeedLimit] = "speedLimit";
   fields[Min]        = "minSpeed";
   fields[Max]        = "maxSpeed";

   return fields;
}
auto TrackSegmentListViewModel::data(const QModelIndex& index, int role) const -> QVariant {
   if (!index.isValid()) {
      return QVariant();
   }

   const int i = index.row();

   auto const& [start, section] = *std::next(track->getSections().begin(), i);

   switch (role) {
      case Start:
         return QVariant(start.to<double>());
      case Length:
         return QVariant(section.length.to<double>());
      case SpeedLimit:
         return QVariant(section.speedLimit.to<double>());
      case Min:
         return QVariant(section.min.to<double>());
      case Max:
         return QVariant(section.max.to<double>());
      default:
         return QVariant();
   }
}

auto TrackSegmentListViewModel::setData(const QModelIndex& index, const QVariant& value, int role) -> bool {
   if (index.isValid() && role == SpeedLimit) {
      const int i             = index.row();
      auto&& [start, section] = *std::next(track->getSections().begin(), i);

      section.speedLimit = si::kilometers_per_hour_t(value.value<double>());
      emit dataChanged(index, index, {Qt::EditRole, Qt::DisplayRole});
      return true;
   }

   return false;
}

} // namespace ngt::speedlimits
