#pragma once

#include <QtCore/QAbstractListModel>
#include <ngt/views/speed-limits/model/Track.hpp>

namespace ngt::speedlimits {

class TrackSegmentListViewModel : public QAbstractListModel {
   Q_OBJECT; // NOLINT

   Q_PROPERTY(double totalLength READ getTotalLength CONSTANT);
   Q_PROPERTY(int numSegments READ numSegments CONSTANT);

 public:
   explicit TrackSegmentListViewModel(Track* track = nullptr);

   TrackSegmentListViewModel(TrackSegmentListViewModel const& other) = delete;
   auto operator=(TrackSegmentListViewModel const& other) -> TrackSegmentListViewModel&;

   TrackSegmentListViewModel(TrackSegmentListViewModel&& other) = delete;
   auto operator=(TrackSegmentListViewModel&& other) noexcept -> TrackSegmentListViewModel&;

   ~TrackSegmentListViewModel() override = default;

   [[nodiscard]]
   auto rowCount(QModelIndex const& parent) const -> int override;

   [[nodiscard]]
   auto data(const QModelIndex& index, int role) const -> QVariant override;

   [[nodiscard]]
   auto setData(const QModelIndex& index, const QVariant& value, int role) -> bool override;

   [[nodiscard]]
   auto roleNames() const -> QHash<int, QByteArray> override;

   [[nodiscard]]
   auto getTotalLength() const -> double { return track->length().to<double>(); }

   [[nodiscard]]
   auto numSegments() const -> int { return static_cast<int>(track->getSections().size()); }

 private:
   Track* track;

   enum { Start = Qt::UserRole, Length, SpeedLimit, Min, Max };
};

} // namespace ngt::speedlimits
