#include "TrainListViewModel.hpp"

namespace ngt::speedlimits {

TrainListViewModel::TrainListViewModel(std::vector<Train> const* trains)
   : QAbstractListModel(nullptr)
   , trains(trains) {}

auto TrainListViewModel::operator=(const TrainListViewModel& other) -> TrainListViewModel& {
   if (this != &other) {
      trains = other.trains;
   }

   return *this;
}

auto TrainListViewModel::operator=(TrainListViewModel&& other) noexcept -> TrainListViewModel& {
   if (this != &other) {
      trains = other.trains;
   }

   return *this;
}

auto TrainListViewModel::rowCount(const QModelIndex& /*parent*/) const -> int {
   return static_cast<int32_t>(trains->size());
}

auto TrainListViewModel::roleNames() const -> QHash<int, QByteArray> {
   QHash<int, QByteArray> fields;
   fields[Position] = "position";
   fields[Speed]    = "speed";
   fields[Length]   = "length";

   return fields;
}

auto TrainListViewModel::data(const QModelIndex& index, int role) const -> QVariant {
   if (!index.isValid()) {
      return QVariant();
   }

   const int i = index.row();

   switch (role) {
      case Position:
         return QVariant(trains->at(i).position.to<double>());
      case Speed:
         return QVariant(static_cast<int>(std::lround(trains->at(i).speed.to<double>())));
      case Length:
         return QVariant(trains->at(i).length.to<int>());
      default:
         return QVariant();
   }
}

void TrainListViewModel::changeEvent() {
   size_t size = trains->size();
   emit   dataChanged(createIndex(0, 0), createIndex(static_cast<int32_t>(size), 0));

   if (lastSize != size) {
      lastSize = size;
      emit layoutChanged();
      emit change();
   }
}

} // namespace ngt::speedlimits