#pragma once

#include <QtCore/QAbstractListModel>
#include <ngt/views/speed-limits/model/Train.hpp>

namespace ngt::speedlimits {

class TrainListViewModel : public QAbstractListModel {
   Q_OBJECT; // NOLINT

   Q_PROPERTY(int numTrains READ numTrains NOTIFY change);

 public:
   explicit TrainListViewModel(std::vector<Train> const* trains = nullptr);

   TrainListViewModel(TrainListViewModel const& other) = delete;
   auto operator                                       =(TrainListViewModel const& other) -> TrainListViewModel&;

   TrainListViewModel(TrainListViewModel&& other) = delete;
   auto operator                                  =(TrainListViewModel&& other) noexcept -> TrainListViewModel&;

   ~TrainListViewModel() override = default;

   [[nodiscard]]
   auto rowCount(QModelIndex const& parent) const -> int override;

   [[nodiscard]]
   auto data(const QModelIndex& index, int role) const -> QVariant override;

   [[nodiscard]]
   auto roleNames() const -> QHash<int, QByteArray> override;

   [[nodiscard]]
   auto numTrains() const -> int { return static_cast<int>(trains->size()); }

   void changeEvent();

 signals:
   void change();

 private:
   std::vector<Train> const* trains;
   size_t                    lastSize = 0;

   enum { Position = Qt::UserRole, Speed, Length };
};

} // namespace ngt::speedlimits
