#pragma once

#include <QtCore/QObject>
#include <ngt/views/speed-limits/model/Train.hpp>

namespace ngt::speedlimits {

class TrainViewModel : public QObject {
   Q_OBJECT; // NOLINT

   Q_PROPERTY(double position READ position NOTIFY change);
   Q_PROPERTY(int speed READ speed NOTIFY change);
   Q_PROPERTY(int length READ length NOTIFY change);

 public:
   explicit TrainViewModel(Train const* train = nullptr)
      : train(train) {}

   TrainViewModel(TrainViewModel const& other) = delete;
   auto operator                               =(TrainViewModel const& other) -> TrainViewModel&;

   TrainViewModel(TrainViewModel&& other) = delete;
   auto operator                          =(TrainViewModel&& other) noexcept -> TrainViewModel&;

   ~TrainViewModel() override = default;

   [[nodiscard]]
   auto position() -> double { return train->position.to<double>(); }

   [[nodiscard]]
   auto speed() -> int { return std::lround(train->speed.to<double>()); }

   [[nodiscard]]
   auto length() -> int { return train->length.to<int>(); }

 signals:
   void change();

 private:
   Train const* train;
};

} // namespace ngt::speedlimits